/* CaptainDomo - the Java mailing list server
 *
 * Pop3.java
 *
 * Copyright (c) 1997 by Xavier Machenaud. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies.
 *
 * Use at your own risk. I do not guarantee the fitness of
 * CaptainDomo for any purpose, and I do not accept responsibility
 * for any damage you do to yourself or others by using CaptainDomo.
 */

import java.io.*;
import java.net.*;
import java.util.*;


class TimeoutException extends Exception {

	TimeoutException() { super(); }


	TimeoutException(String s) { super(s); }

}

/**
 * see RFC 1939 "Post Office Protocol - Version 3"
 */
public class Pop3 {
/**
 * Pop3 server and user info
 */
	protected String Server = null;
	protected int Port= -1;
	protected String User = null;
	protected String Password = null;
/**
 * The socket used to connect to the POP3 server.
 */
  protected Socket sock=null;
/**
 * The input stream from the POP3 server.
 */
  protected DataInputStream in=null;
/**
 * The output stream to the POP3 server.
 */
  protected PrintStream out=null;
/**
 * The last line read from the POP3 server.
 */
  protected String line=null;

  public Pop3(String pServer, int pPort, String pUser, String pPassword) {
	  Server = pServer;
	  Port = pPort;
	  User = pUser;
	  Password = pPassword;
  }

  public boolean connect(){
    //state.status("Contacting "+Server+" on port "+Port+"...");
    try {
		sock=new Socket(Server, Port);
	}
    catch(Exception e) {
      //state.status("Socket connection to "+Server+" on port "+Port+" failed ("+e+")!");
      return false;
    }
    try{
      in=new DataInputStream(new BufferedInputStream(sock.getInputStream()));
      out=new PrintStream(new BufferedOutputStream(sock.getOutputStream()));
    } catch(IOException e) {
      //state.status("Could not open streams to "+Server+"!");
		close();
		return false;
    }
    //state.status("Connected to POP3 socket on "+Server+".");

    do {
		try {
			getLine();
		}
		catch (Exception e) {
			try {
				sock.close();
			}
			catch(IOException ioe) {
				//state.status("Socket close error!");
			}
		return false;
		}
	} while(line==null || !line.regionMatches(0,"+OK",0,3));

	if (line==null) {
		try {
			sock.close();
		}
		catch(IOException e) {
			//state.status("Socket close error!");
		}
		return false;
	}

    out.print("USER "+User+"\r\n");
    out.flush();
	//System.out.println("POP <- USER "+User);

	try {
		getLine();
	}
	catch (Exception e) {
		close();
		return false;
	}

    if(!line.regionMatches(0,"+OK",0,3)){
      //state.status("Retrieval error: User wasn't accepted!");
      close();
      return false;
    }

    out.print("PASS "+Password+"\r\n");
    out.flush();
	//System.out.println("POP <- PASS "+Password);

	try {
		getLine();
	}
	catch (Exception e) {
		close();
		return false;
	}

    if(line==null || !line.regionMatches(0,"+OK",0,3)){
      //state.status("Retrieval error: Password wasn't accepted!");
      close();
      return false;
    }

	return true;
  }


/**
 * Closes the connection to the POP3 server.
 */
  public void close(){
	out.print("QUIT\r\n");
    out.flush();
	//System.out.println("POP <- QUIT");

	try {
		getLine();
	}
	catch (Exception e) {
	// We don't care about the result
	// But if the result isn't OK,
	// some delete could be not processed
	}

    try {
		sock.close();
	}
    catch(IOException e) {
		//state.status("Socket close error!");
		return;
	}
  }

  public InternetTextMessage getMsg(int Id){
	InternetTextMessage message = new InternetTextMessage();

	message.setText(RETR(Id));

	return message;
  }


  public int getMsgCount() {
	return STAT();
  }


  public boolean deleteMsg(int Id) {
	return DELE(Id);
  }

/**
 * The POP3 STAT command
 */
  protected int STAT(){
    out.print("STAT\r\n");
    out.flush();
	//System.out.println("POP <- STAT");

	try {
		getLine();
	}
	catch (Exception e) {
		close();
		return -1;
	}

    if(!line.regionMatches(0,"+OK",0,3)){
      //state.status("Retrieval error: POP3 server error on STAT!");
      close();
      return -1;
    }
    return Integer.parseInt(line.substring(4,line.indexOf(' ',4)));
  }

/**
 * The POP3 RETR command
 */
  protected String RETR(int messageNumber){
	StringBuffer message = new StringBuffer("");
    out.print("RETR "+messageNumber+"\r\n");
    out.flush();
	//System.out.println("POP <- RETR "+messageNumber);

	try {
		getLine();
	}
	catch (Exception e) {
		close();
		return null;
	}

    if(line.regionMatches(0,"+OK",0,3)){
		try {
			getLine();
		}
		catch (Exception e) {
			close();
			return null;
		}

		while(!line.equals(".")) {
			message.append(line).append('\n');
			try {
				getLine();
			}
			catch (Exception e) {
				close();
				return null;
			}
		}
		return message.toString();
	}
	else {
		return null;
	}
  }

/**
 * The POP3 DELE command
 */
  protected boolean DELE(int messageNumber){
	out.print("DELE "+messageNumber+"\r\n");
	out.flush();
	//System.out.println("POP <- DELE "+messageNumber);

	try {
		getLine();
	}
	catch (Exception e) {
		close();
		return false;
	}

	return (line.regionMatches(0,"+OK",0,3));
  }

/**
 * Returns a line read from the POP3 server.  This is a method simply to
 * avoid numerous or unclear try/catch sequences.
 * Because the is not timeout in JDK 1.0.2 Socket, we look if there are
 * characters in the input, wait for a moment if there aren't, look again
 * and throw a TimeOutException if there are still no characters.
 * @return      The line read from the server.
 */
  private String getLine() throws TimeoutException {
    try{
		for (int i=0; i<10 ; i++) {
			if (in.available()!=0)
				break;

			Thread.currentThread().sleep(1000);	// 1 second
		}

		if (in.available()==0) {
			//System.err.println("Throw TimeoutException");
			throw new TimeoutException("Timeout while reading socket");
		}
		else {
			line = in.readLine();
		}
    }
    catch(IOException e){
		//System.err.println("Catch IOException");
      //state.status("Read error!");
      return null;
    }
	catch (InterruptedException e) {
		//System.err.println("Catch InterruptedException");
		return null;
	}

	//System.out.println("POP -> "+line);
    return line;
  }
}
